# Tests the callback used to forward events from the alert library to
# the database writer.

import copy
import datetime
import unittest
from Queue import Queue

from fake import FakeAlert, generate_alerts, generate_alert

from pipevis import AlertListener


class TestListener(unittest.TestCase):
    def test_listener(self):
        # The listener passes alerts to the queue
        queue = Queue()
        out_alerts = []
        listener = AlertListener(queue=queue)
        in_alerts = generate_alerts(num_alerts=10)
        for alert in in_alerts:
            listener(alert)
            out_alerts.append(queue.get())
        self.assertTrue(in_alerts == out_alerts)

    def test_writer_error(self):
        # The listener propogates exceptions from the queue up.
        errmsg = "something bad happened"
        class BadQueue(object):
            def put(self, *args, **kargs):
                raise RuntimeError(errmsg)
        listener = AlertListener(queue=BadQueue())
        alert = generate_alert()
        try:
            listener(alert)
            raise AssertionError("Error expected, but not raised from writer")
        except RuntimeError, e:
            self.assertTrue(str(e) == errmsg)


if __name__ == '__main__':
    unittest.main()
